import requests
import os
import subprocess
import time
from difflib import SequenceMatcher

PROXY_JAR = "infosec2019.jar"
PROXY_CMD = "java -jar " + PROXY_JAR
PROXY_ARG = ''
PROXY_PORT = 8080
PROXY_SHOW_OUT = False

def start_proxy():
    kill_port()
    proxy = subprocess.Popen('exec ' + PROXY_CMD + ' ' + PROXY_ARG, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    time.sleep(2)
    return proxy

def end_proxy(proxy):
    proxy.kill()
    kill_port()
    stdout, stderr = proxy.communicate()
    if PROXY_SHOW_OUT:
        if stdout != None and len(stdout) > 0:
            print('- Proxy stdout:')
            print(stdout.decode('utf-8'))
        if stderr != None and len(stderr) > 0:
            print('- Proxy stderr:')
            print(stderr.decode('utf-8'))

def kill_port():
    # kill all programs listening on PROXY_PORT
    subprocess.Popen('fuser -k {:d}/tcp'.format(PROXY_PORT), shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)


def get_page(url, with_proxy=False):
    proxies = {}
    if with_proxy:
        proxies = {
            'http': 'http://localhost:{:d}'.format(PROXY_PORT),
            'https': 'http://localhost:{:d}'.format(PROXY_PORT),
        }

    # Attention: python requests/httplib uses HTTP 1.0 for HTTP CONNECT, and therefore does not send a Host header, this is not common but valid!
    # If you do not want to implement this case, either use curl/browser for testing instead of requests for HTTPS tests
    try:
        response = requests.get(url, proxies=proxies)
        return response.text
    except Exception as e:
        print(' -- Got exception')
        return b''

def test_proxy(name, url):
    print('Testing {} ({})'.format(name, url))

    start = time.time()
    response_normal = get_page(url)
    end = time.time()
    time_normal = end-start
    p = start_proxy()
    start = time.time()
    response_proxy  = get_page(url, with_proxy=True)
    end = time.time()
    time_proxy = end-start
    end_proxy(p)

    print(' - Request time (normal): {:.3f}s'.format(time_normal))
    print(' - Request time (proxy):  {:.3f}s'.format(time_proxy))
    print(' - Response length (normal): {}'.format(len(response_normal)))
    print(' - Response length (proxy) : {}'.format(len(response_proxy)))
    print(' - Similarity: {:f}'.format(SequenceMatcher(a=response_normal, b=response_proxy).ratio()))


test_urls = {
    'httpbin' : 'http://httpbin.org',
    'httpbin (HTTPS)' : 'https://httpbin.org',
    'http chunked encoding' : 'http://www.httpwatch.com/httpgallery/chunked/',
}

if __name__ == '__main__':

    for name, url in test_urls.items():
        test_proxy(name, url)

